from __future__ import annotations

import importlib.metadata

from packaging.version import Version

import idc_index_data as m

EXPECTED_IDC_INDEX_VERSION = 17


def test_version():
    assert importlib.metadata.version("idc_index_data") == m.__version__


def test_idc_index_version():
    assert Version(m.__version__).major == EXPECTED_IDC_INDEX_VERSION


def test_filepath():
    if m.IDC_INDEX_CSV_ARCHIVE_FILEPATH is not None:
        assert m.IDC_INDEX_CSV_ARCHIVE_FILEPATH.is_file()
        assert m.IDC_INDEX_CSV_ARCHIVE_FILEPATH.name == "idc_index.csv.zip"

    if m.IDC_INDEX_PARQUET_FILEPATH is not None:
        assert m.IDC_INDEX_PARQUET_FILEPATH.is_file()
        assert m.IDC_INDEX_PARQUET_FILEPATH.name == "idc_index.parquet"



"""
Copyright (c) 2024 Andrey Fedorov. All rights reserved.

idc-index-data: ImagingDataCommons index to query and download data.
"""

from __future__ import annotations

from importlib.metadata import distribution
from pathlib import Path

from ._version import version as __version__

__all__ = [
    "__version__",
    "IDC_INDEX_CSV_ARCHIVE_FILEPATH",
    "IDC_INDEX_PARQUET_FILEPATH",
]


def _lookup(path: str, optional: bool = False) -> Path | None:
    """Support editable installation by looking up path using distribution API."""
    files = distribution("idc_index_data").files
    if files is not None:
        for _file in files:
            if str(_file) == path:
                return Path(str(_file.locate())).resolve(strict=True)
    if optional:
        return None

    msg = f"Failed to lookup '{path}`."
    raise FileNotFoundError(msg)


IDC_INDEX_CSV_ARCHIVE_FILEPATH: Path | None = _lookup(
    "idc_index_data/idc_index.csv.zip", optional=True
)
IDC_INDEX_PARQUET_FILEPATH: Path | None = _lookup("idc_index_data/idc_index.parquet")



#!/usr/bin/env python3
"""
Command line executable allowing to update source files given a IDC index version.
"""

from __future__ import annotations

import argparse
import contextlib
import os
import re
import textwrap
from pathlib import Path

ROOT_DIR = Path(__file__).parent / "../.."


@contextlib.contextmanager
def _log(txt, verbose=True):
    if verbose:
        print(txt)  # noqa: T201
    yield
    if verbose:
        print(f"{txt} - done")  # noqa: T201


def _update_file(filepath, regex, replacement):
    msg = "Updating %s" % os.path.relpath(str(filepath), ROOT_DIR)
    with _log(msg):
        pattern = re.compile(regex)
        with filepath.open() as doc_file:
            lines = doc_file.readlines()
            updated_content = []
            for line in lines:
                updated_content.append(re.sub(pattern, replacement, line))
        with filepath.open("w") as doc_file:
            doc_file.writelines(updated_content)


def update_pyproject_toml(idc_index_version):
    pattern = re.compile(r'^version = "[\w\.]+"$')
    replacement = f'version = "{idc_index_version}.0.0"'
    _update_file(ROOT_DIR / "pyproject.toml", pattern, replacement)


def update_sql_scripts(idc_index_version):
    pattern = re.compile(r"idc_v\d+")
    replacement = f"idc_v{idc_index_version}"
    _update_file(ROOT_DIR / "scripts/sql/idc_index.sql", pattern, replacement)


def update_tests(idc_index_version):
    pattern = re.compile(r"EXPECTED_IDC_INDEX_VERSION = \d+")
    replacement = f"EXPECTED_IDC_INDEX_VERSION = {idc_index_version}"
    _update_file(ROOT_DIR / "tests/test_package.py", pattern, replacement)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "idc_index_version",
        metavar="IDC_INDEX_VERSION",
        type=int,
        help="IDC index version of the form NN",
    )
    parser.add_argument(
        "--quiet",
        action="store_true",
        help="Hide the output",
    )

    args = parser.parse_args()

    update_pyproject_toml(args.idc_index_version)
    update_sql_scripts(args.idc_index_version)
    update_tests(args.idc_index_version)

    if not args.quiet:
        msg = """\
            Complete! Now run:

            git switch -c update-to-idc-index-{release}
            git add -u pyproject.toml scripts/sql/idc_index.sql tests/test_package.py
            git commit -m "Update to IDC index {release}"
            gh pr create --fill --body "Created by update_idc_index_version.py"
            """
        print(textwrap.dedent(msg.format(release=args.idc_index_version)))  # noqa: T201


if __name__ == "__main__":
    main()



from __future__ import annotations

import logging
import os
from pathlib import Path

import pandas as pd
from google.cloud import bigquery

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)


class IDCIndexDataManager:
    def __init__(self, project_id: str):
        """
        Initializes the IDCIndexDataManager using the Google Cloud Platform project ID.
        """
        self.project_id = project_id
        self.client = bigquery.Client(project=project_id)
        logger.debug("IDCIndexDataManager initialized with project ID: %s", project_id)

    def execute_sql_query(self, file_path: str) -> tuple[pd.DataFrame, str]:
        """
        Executes the SQL query in the specified file.

        Returns:
            Tuple[pd.DataFrame, str]: A tuple containing the DataFrame with query results,
            the output basename.
        """
        with Path(file_path).open("r") as file:
            sql_query = file.read()
        index_df = self.client.query(sql_query).to_dataframe()
        output_basename = Path(file_path).name.split(".")[0]
        logger.debug("Executed SQL query from file: %s", file_path)
        return index_df, output_basename

    def generate_index_data_files(
        self, generate_compressed_csv: bool = True, generate_parquet: bool = False
    ) -> None:
        """
        Generates index-data files locally by executing queries against
        the Google Cloud Platform IDC project tables.

        This method iterates over SQL files in the 'scripts/sql' directory,
        executing each query using :func:`execute_sql_query` and generating a DataFrame,
        'index_df'. The DataFrame is then saved as compressed CSV and/or Parquet file.
        """

        scripts_dir = Path(__file__).parent.parent
        sql_dir = scripts_dir / "sql"

        for file_name in os.listdir(sql_dir):
            if file_name.endswith(".sql"):
                file_path = Path(sql_dir) / file_name
                index_df, output_basename = self.execute_sql_query(file_path)
                logger.debug(
                    "Executed and processed SQL queries from file: %s", file_path
                )
            if generate_compressed_csv:
                csv_file_name = f"{output_basename}.csv.zip"
                index_df.to_csv(
                    csv_file_name, compression={"method": "zip"}, escapechar="\\"
                )
                logger.debug("Created CSV zip file: %s", csv_file_name)

            if generate_parquet:
                parquet_file_name = f"{output_basename}.parquet"
                index_df.to_parquet(parquet_file_name, compression="zstd")
                logger.debug("Created Parquet file: %s", parquet_file_name)

    def retrieve_latest_idc_release_version(self) -> int:
        """
        Retrieves the latest IDC release version.

        This function executes a SQL query on the `version_metadata` table in the
        `idc_current` dataset of the BigQuery client. It retrieves the maximum
        `idc_version` and returns it as an integer.
        """
        query = """
        SELECT
            MAX(idc_version) AS latest_idc_release_version
        FROM
            `bigquery-public-data.idc_current.version_metadata`
        """
        query_job = self.client.query(query)
        result = query_job.result()
        return int(next(result).latest_idc_release_version)


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--project",
        default=os.environ.get("GCP_PROJECT", None),
        help="Google Cloud Platform Project ID (default from GCP_PROJECT env. variable)",
    )
    parser.add_argument(
        "--generate-csv-archive",
        action="store_true",
        help="Generate idc_index.csv.zip file",
    )
    parser.add_argument(
        "--generate-parquet",
        action="store_true",
        help="Generate idc_index.parquet file",
    )
    parser.add_argument(
        "--retrieve-latest-idc-release-version",
        action="store_true",
        help="Retrieve and display the latest IDC release version",
    )

    args = parser.parse_args()

    if not args.project:
        parser.error(
            "Set GCP_PROJECT environment variable or specify --project argument"
        )

    if any([args.generate_csv_archive, args.generate_parquet]):
        IDCIndexDataManager(args.project).generate_index_data_files(
            generate_compressed_csv=args.generate_csv_archive,
            generate_parquet=args.generate_parquet,
        )
    elif args.retrieve_latest_idc_release_version:
        logging.basicConfig(level=logging.ERROR, force=True)
        logger.setLevel(logging.ERROR)
        version = IDCIndexDataManager(
            args.project
        ).retrieve_latest_idc_release_version()
        print(f"{version}")  # noqa: T201
    else:
        parser.print_help()



from __future__ import annotations

import argparse
import re
import shutil
from pathlib import Path

import nox

DIR = Path(__file__).parent.resolve()

nox.options.sessions = ["lint", "pylint", "tests"]  # Session run by default


@nox.session
def lint(session: nox.Session) -> None:
    """
    Run the linter.
    """
    session.install("pre-commit")
    session.run(
        "pre-commit", "run", "--all-files", "--show-diff-on-failure", *session.posargs
    )


@nox.session
def pylint(session: nox.Session) -> None:
    """
    Run PyLint.
    """
    # This needs to be installed into the package environment, and is slower
    # than a pre-commit check
    session.install(".", "pylint")
    session.run("pylint", "idc_index_data", *session.posargs)


@nox.session
def tests(session: nox.Session) -> None:
    """
    Run the unit and regular tests.
    """
    session.install(".[test]")
    session.run("pytest", *session.posargs)


@nox.session(reuse_venv=True)
def docs(session: nox.Session) -> None:
    """
    Build the docs. Pass "--serve" to serve. Pass "-b linkcheck" to check links.
    """

    parser = argparse.ArgumentParser()
    parser.add_argument("--serve", action="store_true", help="Serve after building")
    parser.add_argument(
        "-b", dest="builder", default="html", help="Build target (default: html)"
    )
    args, posargs = parser.parse_known_args(session.posargs)

    if args.builder != "html" and args.serve:
        session.error("Must not specify non-HTML builder with --serve")

    extra_installs = ["sphinx-autobuild"] if args.serve else []

    session.install("-e.[docs]", *extra_installs)
    session.chdir("docs")

    if args.builder == "linkcheck":
        session.run(
            "sphinx-build", "-b", "linkcheck", ".", "_build/linkcheck", *posargs
        )
        return

    shared_args = (
        "-n",  # nitpicky mode
        "-T",  # full tracebacks
        f"-b={args.builder}",
        ".",
        f"_build/{args.builder}",
        *posargs,
    )

    if args.serve:
        session.run("sphinx-autobuild", *shared_args)
    else:
        session.run("sphinx-build", "--keep-going", *shared_args)


@nox.session
def build_api_docs(session: nox.Session) -> None:
    """
    Build (regenerate) API docs.
    """

    session.install("sphinx")
    session.chdir("docs")
    session.run(
        "sphinx-apidoc",
        "-o",
        "api/",
        "--module-first",
        "--no-toc",
        "--force",
        "../src/idc_index_data",
    )


@nox.session
def build(session: nox.Session) -> None:
    """
    Build an SDist and wheel.
    """

    build_path = DIR.joinpath("build")
    if build_path.exists():
        shutil.rmtree(build_path)

    session.install("build")
    session.run("python", "-m", "build")


def _bump(session: nox.Session, name: str, script: str, files) -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--commit", action="store_true", help="Make a branch and commit."
    )
    parser.add_argument(
        "version", nargs="?", help="The version to process - leave off for latest."
    )
    args = parser.parse_args(session.posargs)

    session.install("db-dtypes")
    session.install("google-cloud-bigquery")
    session.install("pandas")
    session.install("pyarrow")

    if args.version is None:
        gcp_project = "idc-external-025"
        idc_index_version = session.run(
            "python",
            "scripts/python/idc_index_data_manager.py",
            "--project",
            gcp_project,
            "--retrieve-latest-idc-release-version",
            external=True,
            silent=True,
        ).strip()

    else:
        idc_index_version = args.version

    extra = ["--quiet"] if args.commit else []
    session.run("python", script, idc_index_version, *extra)

    if args.commit:
        session.run(
            "git",
            "switch",
            "-c",
            f"update-to-{name.replace(' ', '-').lower()}-{idc_index_version}",
            external=True,
        )
        session.run("git", "add", "-u", *files, external=True)
        session.run(
            "git",
            "commit",
            "-m",
            f"Update to {name} {idc_index_version}",
            external=True,
        )
        session.log(
            f'Complete! Now run: gh pr create --fill --body "Created by running `nox -s {session.name} -- --commit`"'
        )


@nox.session
def bump(session: nox.Session) -> None:
    """
    Set to a new IDC index version, use -- <version>, otherwise will use the latest version.
    """
    files = (
        "pyproject.toml",
        "scripts/sql/idc_index.sql",
        "tests/test_package.py",
    )
    _bump(
        session,
        "IDC index",
        "scripts/python/update_idc_index_version.py",
        files,
    )


@nox.session(venv_backend="none")
def tag_release(session: nox.Session) -> None:
    """
    Print instructions for tagging a release and pushing it to GitHub.
    """

    session.log("Run the following commands to make a release:")
    txt = Path("pyproject.toml").read_text()
    current_version = next(iter(re.finditer(r'^version = "([\d\.]+)$"', txt))).group(1)
    print(
        f"git tag --sign -m 'idc-index-data {current_version}' {current_version} main"
    )
    print(f"git push origin {current_version}")
